import { SerializedError } from "@reduxjs/toolkit";
import { FetchBaseQueryError } from "@reduxjs/toolkit/dist/query";
import { ReactElement } from "react";
export declare type UseQueryResult<T> = {
    originalArgs?: unknown;
    data?: T;
    currentData?: T;
    error?: unknown;
    requestId?: string;
    endpointName?: string;
    startedTimeStamp?: number;
    fulfilledTimeStamp?: number;
    isUninitialized: boolean;
    isLoading: boolean;
    isFetching: boolean;
    isSuccess: boolean;
    isError: boolean;
    refetch: () => void;
};
export declare type LoaderTransformFunction<QRU extends readonly UseQueryResult<unknown>[], R extends unknown> = (queries: QRU) => R;
export declare type OptionalGenericArg<T> = T extends never ? [] : [T];
export declare type CreateUseLoaderArgs<QRU extends readonly UseQueryResult<unknown>[], R extends unknown, A = never> = {
    queries: (...args: OptionalGenericArg<A>) => QRU;
    transform?: LoaderTransformFunction<QRU, R>;
};
export declare type UseLoader<A, R> = (...args: OptionalGenericArg<A>) => UseQueryResult<R>;
export declare type CreateLoaderType = <QRU extends readonly UseQueryResult<unknown>[], R extends unknown = QRU, A = never>(createLoaderArgs: CreateUseLoaderArgs<QRU, R, A>) => UseLoader<A, R>;
export declare type ComponentWithLoaderData<P extends Record<string, any>, R extends unknown> = (props: P, loaderData: R) => ReactElement;
declare type InferQueryResult<T> = T extends UseQueryResult<infer X> ? X : never;
export declare type InferLoaderData<T> = T extends UseLoader<any, any> ? InferQueryResult<ReturnType<T>> : never;
export declare type WithLoaderArgs<P extends unknown, R extends unknown, A = never> = {
    useLoader: UseLoader<A, R>;
    useLoaderArg?: (props: P) => A;
    onLoading?: (props: P) => ReactElement;
    onError?: (props: P, error: SerializedError | FetchBaseQueryError) => ReactElement;
    onFetching?: (props: P) => ReactElement;
};
export declare type Component<P extends Record<string, unknown>> = (props: P) => ReactElement;
export declare type Loader<P extends unknown, R extends unknown, A = never> = WithLoaderArgs<P, R, A>;
export {};
